package net.h34t.filemure.core.entity

import java.time.LocalDateTime

data class Document(
    val id: Long,
    val extId: String,
    val title: String,
    val description: String,
    val tags: List<Tag>,
    val created: LocalDateTime,
    val referenceDate: LocalDateTime,
    val state: State,
    val files: List<FileRef>,
)

@JvmInline
value class Tag(val value: String) {
    init {
        // TODO proper validation
        require(value.isNotBlank())
    }

    companion object {
        fun parse(ser: String?): List<Tag> {
            return ser?.let { if (it.isNotBlank()) it.split(",").map { Tag(it) } else emptyList() } ?: emptyList()
        }

        fun List<Tag>.serialize() = if (this.isEmpty()) "" else this.joinToString(",") { it.value }
    }
}

data class FileRef(
    val id: Long,
    val extId: String,
    val accountId: Long,
    val documentId: Long?,
    val filename: String,
    val contentType: String?,
    val contentExtracted: String?,
    val fileSize: Long,
    val created: LocalDateTime,
    val state: State
)

data class FileContent(
    val id: Long,
    val extId: String,
    val filename: String,
    val contentType: String?,
    val contentExtracted: String?,
    val fileSize: Long,
    val content: ByteArray,
) {
    override fun equals(other: Any?): Boolean {
        if (this === other) return true
        if (javaClass != other?.javaClass) return false

        other as FileContent

        if (id != other.id) return false
        if (extId != other.extId) return false
        if (!content.contentEquals(other.content)) return false

        return true
    }

    override fun hashCode(): Int {
        var result = id.hashCode()
        result = 31 * result + extId.hashCode()
        result = 31 * result + content.contentHashCode()
        return result
    }
}

enum class State(val code: Int) {
    ACTIVE(1),
    ARCHIVED(2),
    DELETED(3);

    companion object {
        fun fromCode(code: Int) = when (code) {
            1 -> ACTIVE
            2 -> ARCHIVED
            3 -> DELETED
            else -> throw IllegalArgumentException("$code doesn't exist.")
        }
    }
}
